#!/usr/bin/perl -w
#
# bw-contact
#
# Copyright (c) 1995-2003 William E. Weinman
# All rights reserved worldwide. 
#
# This program is free software; you can redistribute it and/or modify it
# under the same terms as Perl itself. You must retain this copyright 
# notice, and if you redistribute a modified version you must insert a 
# prominent notice in each changed file stating how and when you changed 
# that file, and make your changes freely available according to the 
# terms of the "Artistic License" distributed with Perl. 
#
# #####
#
# This program provides a form for sending email to the owner of 
# a web site. It uses external HTML files to provide a look
# consistent the rest of the web site. 
#
# Requires modules bwConfig, bwInclude, and bwEmail. 
#
# version 1.02 -- 2003-08-05 -- wew --
#   added code to make sure name part of return address is in quotes
#
# version 1.01 -- 2003-06-26 -- wew --
#   bugfix: some servers don't pass REQUEST_URI so this version
#     uses either REQUEST_URI or SCRIPT_NAME
#   feature: now allows use of sendmail if smtp is not available
#     to use sendmail, put path to sendmail in "sendmial" option 
#     in config, and comment out the "smtp" option. 
#

##### ----- ----- ----- ----- ----- ----- ----- ----- ----- -----
#
# Enter the path to your config file 
#
my $config_path = "bw-contact.conf";


# uncomment the following and enter the path to your bw libraries 
# if neccessary
# 
use lib ("./");

#
# Nothing below this line should need modification. 
#
##### ----- ----- ----- ----- ----- ----- ----- ----- ----- -----

use CGI;
use bwInclude;
use bwEmail;
use bwConfig;

my $TRUE = 1;
my $FALSE = '';

use subs qw( error );

# global variables
my $g = {

  # configuration file
  config_file => $config_path,

  # sign on values
  progname    => "bw-contact",
  version     => "1.02",
  byline      => "Bill Weinman",
  url         => "http://bw.org/",
  copyright   => "&copy; 1995-2003 William E. Weinman",
  0 => 0

  };

my $q = new CGI;
my $qv = $q->Vars;
my $p = new bwInclude;
$p->quiet($TRUE); # supress "Undefined Variable" messages

# create the callback URI
my $_self = $ENV{REQUEST_URI} || $ENV{SCRIPT_NAME} || '';
$_self =~ s/[\?\#].*//;
$p->var('SELF', $_self);

read_config();

# jump table
if(my $cmd = $qv->{cmd}) {
  show_config()  if $cmd eq  'show_config';
  firstpage()    if $cmd eq  'firstpage';
  confirm()      if $cmd eq  'confirm';
  send_message() if $cmd eq  'send';
  show_config();   # default
  }

# fall through
firstpage();

exit;

###

sub read_config
{
my $config = new bwConfig($g->{config_file});
my $cfe = $config->error;
error "$cfe\n" if $cfe;
$g->{CONFIG} = $config;
$g->{CV} = $config->values;
$g->{ERROR_PAGE} = $g->{CV}{htmlerror};
}

sub firstpage
{
my $referer = $qv->{Referer} || $ENV{HTTP_REFERER} || '(none)';
if(my $firstpage = $g->{CV}{htmlfirst}) {
  $p->var('Efromname',  $q->escapeHTML($qv->{Efromname})) if $qv->{Efromname};
  $p->var('Efromemail', $q->escapeHTML($qv->{Efromemail})) if $qv->{Efromemail};
  $p->var('Esubject',   $q->escapeHTML($qv->{Esubject})) if $qv->{Esubject};
  $p->var('Emessage',   $q->escapeHTML($qv->{Emessage})) if $qv->{Emessage};
  $p->var('HIDDENS', hiddens( { Referer => $referer, cmd => 'confirm' } ));
  send_header();
  $p->pf($firstpage);
  }
else { 
  error "htmlfirst must be defined in $g->{config_file}" 
  }
exit;
}

sub confirm
{
error_confirm("The email address you entered ($qv->{Efromemail}) is not valid.") 
  unless bwEmail::validate_email($qv->{Efromemail});
error_confirm("You must enter both your name and your email address.") unless $qv->{Efromname} =~ /\S/;
error_confirm("You must supply a subject for your message.") unless $qv->{Esubject} =~ /\S/;
error_confirm("The message is empty. Please enter your message.") unless $qv->{Emessage} =~ /\S/;

firstpage() if error_confirm();

if(my $confpage = $g->{CV}{htmlconfirm}) {
  # wrap Efromname in quotes, if necessary
  $qv->{qEfromname} = ($qv->{Efromname} =~ /"/) ? $qv->{Efromname} : q|"| . $qv->{Efromname} . q|"| 
    if $qv->{Efromname};

  $p->var('Efromname',  $q->escapeHTML($qv->{qEfromname})) if $qv->{Efromname};
  $p->var('Efromemail', $q->escapeHTML($qv->{Efromemail})) if $qv->{Efromemail};
  $p->var('Esubject',   $q->escapeHTML($qv->{Esubject})) if $qv->{Esubject};
  $p->var('Emessage',   $q->escapeHTML($qv->{Emessage})) if $qv->{Emessage};
  $p->var('HIDDENS', hiddens( { 
    Efromname   => $qv->{Efromname}, 
    Efromemail  => $qv->{Efromemail}, 
    Esubject    => $qv->{Esubject}, 
    Emessage    => $qv->{Emessage},
    Referer     => $qv->{Referer}
    } ));
  send_header();
  $p->pf($confpage);
  }
else { 
  error "htmlconfirm must be defined in $g->{config_file}" 
  }
exit;
}

sub error_confirm
{
my $pverr = $p->var('CONFIRM_ERROR');
if(@_) {
  my $err = shift;
  $pverr .= "<br>\n" if $pverr;
  $err = qq{<span style="color: red; font-weight: bold">$err</span>};
  $err = $pverr . $err if $pverr;
  $pverr = $p->var('CONFIRM_ERROR', $err);
  }
return $pverr;
}

sub send_message
{
error "sendmail or smtp_host must be defined in $g->{config_file}" 
  unless ($g->{CV}{smtp_host} || $g->{CV}{sendmail});
if(my $completepage = $g->{CV}{htmlcomplete}) {
  # wrap Efromname in quotes, if necessary
  $qv->{Efromname} = ($qv->{Efromname} =~ /"/) ? $qv->{Efromname} : q|"| . $qv->{Efromname} . q|"| 
    if $qv->{Efromname};

  my $em = new bwEmail;
  $em->header('X-Mailer', "$g->{progname}/$g->{version} (by $g->{byline} <$g->{url}>)");
  $em->header('X-User-Agent', $ENV{HTTP_USER_AGENT});
  $em->header('X-Referer', $qv->{Referer});
  $em->from($qv->{Efromemail});
  $em->from_name($qv->{Efromname});
  $em->to($g->{CV}{toemail});
  $em->to_name($g->{CV}{toname});
  $em->subject($qv->{Esubject});
  $em->body($qv->{Emessage});

  if($g->{CV}{smtp_host}) {
    $em->smtphost($g->{CV}{smtp_host});
    $em->send;
    my $emerr = $em->error;
    error "$emerr\n" if $emerr;
    }
  else {
    my $sendmail = $g->{CV}{sendmail};
    my $message = $em->message;
    
    open(SENDMAIL, "|$sendmail -t -f $qv->{Efromemail}") or error "cannot open $sendmail\n";
    print SENDMAIL $message;
    }

  $p->var('Efromname',  $q->escapeHTML($qv->{Efromname})) if $qv->{Efromname};
  $p->var('Efromemail', $q->escapeHTML($qv->{Efromemail})) if $qv->{Efromemail};
  $p->var('Esubject',   $q->escapeHTML($qv->{Esubject})) if $qv->{Esubject};
  $p->var('Emessage',   $q->escapeHTML($qv->{Emessage})) if $qv->{Emessage};

  send_header();
  $p->pf($completepage);
  }
else { 
  error "htmlcomplete must be defined in $g->{config_file}" 
  }
exit;
}

sub hiddens
{
my $h = shift || return $FALSE;
my $s = '';
foreach my $k ( sort keys %$h ) {
  my $v = $h->{$k} || '';
  $v = $q->escapeHTML($v);
  $s .= qq{<input type=hidden name="$k" value="$v">\n};
  }
return $s;
}

sub show_config
{
send_header();
print <<CONFIG;
<head>
<title> $g->{progname} &middot; show_config </title>
</head>
<body style="background-color: white; font-family: tahoma, helvetica">
<h1 style="font-size: 20px"> $g->{progname} &middot; show_config </h1>
<p> $g->{progname} version $g->{version} by <a href="$g->{url}">$g->{byline}</a><br/>
    $g->{copyright}

<br/><br/>

<table border=1 cellpadding=2 cellspacing=0>
  <caption>Module Versions</caption>
  <tr><th>Module     </td><th> Version              </td></tr>
  <tr><td>CGI        </td><td> $CGI::VERSION        </td></tr>
  <tr><td>bwInclude  </td><td> $bwInclude::VERSION  </td></tr>
  <tr><td>bwEmail    </td><td> $bwEmail::VERSION    </td></tr>
  <tr><td>bwConfig   </td><td> $bwConfig::VERSION   </td></tr>
</table>

CONFIG

my $config = $g->{CONFIG};

my $v = $config->values;
my $a = $config->arrays;

print "<p><table cellspacing=0 cellpadding=2 border=1>\n";
print "<caption>Config Variables</caption>\n";
print "<tr><th>key</th><th>value(s)</th></tr>\n";
foreach my $k ( @{$config->keywords} ) {
  print "<tr><td>$k</td><td>";
  # print the array if there is more than one item
  if(@{$a->{$k}} > 1) {
    print join(' / ', @{$a->{$k}});
    }
  # print the value, if set
  else {
    print "$v->{$k}" if $v->{$k};
    }
  print "</td></tr>\n";
  }

print "</table>\n";

# cgi vars
print "<p><table cellspacing=0 cellpadding=2 border=1>\n";
print "<caption>CGI Variables</caption>\n";
print "<tr><th>key</th><th>value</th></tr>\n";
foreach my $k ( sort keys %{$qv} ) {
  print "<tr><td>$k</td><td>$qv->{$k}</td></tr>\n";
  }

print "</table>\n";

exit;
}

sub send_header
{
return if $g->{SENT_HEADER};
$g->{SENT_HEADER} = $TRUE;
print "Robots: noindex,nofollow\n";
print "Content-type: text/html\n\n";
}

sub error
{
my $e = shift or return;
send_header();
if($g->{ERROR_PAGE}) {
  $p->var('errormessage', $e);
  $p->pf($g->{ERROR_PAGE});
  }
else {
  print qq{<p><span style="color: red; font-weight: bold; background-color: #ffc">$e</span></p>\n};
  }
exit;
}

